mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
From: Tony Luck <tony.luck@intel.com>
To: Fenghua Yu <fenghuay@nvidia.com>,
	Reinette Chatre <reinette.chatre@intel.com>,
	Maciej Wieczor-Retman <maciej.wieczor-retman@intel.com>,
	Peter Newman <peternewman@google.com>,
	James Morse <james.morse@arm.com>,
	Babu Moger <babu.moger@amd.com>,
	Drew Fustini <dfustini@baylibre.com>,
	Dave Martin <Dave.Martin@arm.com>,
	Anil Keshavamurthy <anil.s.keshavamurthy@intel.com>,
	Chen Yu <yu.c.chen@intel.com>
Cc: x86@kernel.org, linux-kernel@vger.kernel.org,
	patches@lists.linux.dev, Tony Luck <tony.luck@intel.com>
Subject: [PATCH v7 26/31] fs/resctrl: Fix life-cycle of closid_num_dirty_rmid
Date: Fri, 11 Jul 2025 16:53:33 -0700	[thread overview]
Message-ID: <20250711235341.113933-27-tony.luck@intel.com> (raw)
In-Reply-To: <20250711235341.113933-1-tony.luck@intel.com>

closid_num_dirty_rmid is specific to the L3 resource, but it
is allocated/freed in the more generic dom_data_{init,exit}().

Add helpers to allocate/free closid_num_dirty_rmid.

Rename resctrl_mon_resource_init() to resctrl_mon_l3_resource_init()
and call the closid_num_dirty_rmid_init() here, instead of
allocating in dom_data_init().

Making matching changes to the exit path by renaming
resctrl_mon_resource_exit() to resctrl_mon_l3_resource_exit()
and free closid_num_dirty_rmid here instead of in dom_data_exit().

Suggested-by: Reinette Chatre <reinette.chatre@intel.com>
Signed-off-by: Tony Luck <tony.luck@intel.com>
---
 fs/resctrl/internal.h |  6 ++--
 fs/resctrl/monitor.c  | 69 ++++++++++++++++++++++++-------------------
 fs/resctrl/rdtgroup.c | 12 ++++----
 3 files changed, 48 insertions(+), 39 deletions(-)

diff --git a/fs/resctrl/internal.h b/fs/resctrl/internal.h
index 56fdccb39375..28d505efdb7c 100644
--- a/fs/resctrl/internal.h
+++ b/fs/resctrl/internal.h
@@ -358,7 +358,9 @@ int alloc_rmid(u32 closid);
 
 void free_rmid(u32 closid, u32 rmid);
 
-void resctrl_mon_resource_exit(void);
+int resctrl_mon_l3_resource_init(void);
+
+void resctrl_mon_l3_resource_exit(void);
 
 void mon_event_count(void *info);
 
@@ -368,8 +370,6 @@ void mon_event_read(struct rmid_read *rr, struct rdt_resource *r,
 		    struct rdt_domain_hdr *hdr, struct rdtgroup *rdtgrp,
 		    cpumask_t *cpumask, struct mon_evt *evt, int first);
 
-int resctrl_mon_resource_init(void);
-
 void mbm_setup_overflow_handler(struct rdt_l3_mon_domain *dom,
 				unsigned long delay_ms,
 				int exclude_cpu);
diff --git a/fs/resctrl/rdtgroup.c b/fs/resctrl/rdtgroup.c
index 9e4df213906f..b45f3d63c629 100644
--- a/fs/resctrl/rdtgroup.c
+++ b/fs/resctrl/rdtgroup.c
@@ -4114,7 +4114,7 @@ void resctrl_offline_mon_domain(struct rdt_resource *r, struct rdt_domain_hdr *h
  * Called when the first CPU of a domain comes online, regardless of whether
  * the filesystem is mounted.
  * During boot this may be called before global allocations have been made by
- * resctrl_mon_resource_init().
+ * resctrl_mon_l3_resource_init().
  *
  * Returns 0 for success, or -ENOMEM.
  */
@@ -4298,13 +4298,13 @@ int resctrl_init(void)
 
 	thread_throttle_mode_init();
 
-	ret = resctrl_mon_resource_init();
+	ret = resctrl_mon_l3_resource_init();
 	if (ret)
 		return ret;
 
 	ret = sysfs_create_mount_point(fs_kobj, "resctrl");
 	if (ret) {
-		resctrl_mon_resource_exit();
+		resctrl_mon_l3_resource_exit();
 		return ret;
 	}
 
@@ -4339,7 +4339,7 @@ int resctrl_init(void)
 
 cleanup_mountpoint:
 	sysfs_remove_mount_point(fs_kobj, "resctrl");
-	resctrl_mon_resource_exit();
+	resctrl_mon_l3_resource_exit();
 
 	return ret;
 }
@@ -4375,7 +4375,7 @@ static bool resctrl_online_domains_exist(void)
  * When called by the architecture code, all CPUs and resctrl domains must be
  * offline. This ensures the limbo and overflow handlers are not scheduled to
  * run, meaning the data structures they access can be freed by
- * resctrl_mon_resource_exit().
+ * resctrl_mon_l3_resource_exit().
  *
  * After resctrl_exit() returns, the architecture code should return an
  * error from all resctrl_arch_ functions that can do this.
@@ -4402,5 +4402,5 @@ void resctrl_exit(void)
 	 * it can be used to umount resctrl.
 	 */
 
-	resctrl_mon_resource_exit();
+	resctrl_mon_l3_resource_exit();
 }
diff --git a/fs/resctrl/monitor.c b/fs/resctrl/monitor.c
index 92798e1fb5b0..e3eceba70713 100644
--- a/fs/resctrl/monitor.c
+++ b/fs/resctrl/monitor.c
@@ -86,6 +86,37 @@ unsigned int resctrl_rmid_realloc_threshold;
  */
 unsigned int resctrl_rmid_realloc_limit;
 
+static int closid_num_dirty_rmid_init(struct rdt_resource *r)
+{
+	if (IS_ENABLED(CONFIG_RESCTRL_RMID_DEPENDS_ON_CLOSID) &&
+	    !closid_num_dirty_rmid) {
+		u32 num_closid = resctrl_arch_get_num_closid(r);
+		u32 *tmp;
+
+		/*
+		 * If the architecture hasn't provided a sanitised value here,
+		 * this may result in larger arrays than necessary. Resctrl will
+		 * use a smaller system wide value based on the resources in
+		 * use.
+		 */
+		tmp = kcalloc(num_closid, sizeof(*tmp), GFP_KERNEL);
+		if (!tmp)
+			return -ENOMEM;
+
+		closid_num_dirty_rmid = tmp;
+	}
+
+	return 0;
+}
+
+static void closid_num_dirty_rmid_exit(void)
+{
+	if (IS_ENABLED(CONFIG_RESCTRL_RMID_DEPENDS_ON_CLOSID)) {
+		kfree(closid_num_dirty_rmid);
+		closid_num_dirty_rmid = NULL;
+	}
+}
+
 /*
  * x86 and arm64 differ in their handling of monitoring.
  * x86's RMID are independent numbers, there is only one source of traffic
@@ -805,36 +836,14 @@ void mbm_setup_overflow_handler(struct rdt_l3_mon_domain *dom, unsigned long del
 static int dom_data_init(struct rdt_resource *r)
 {
 	u32 idx_limit = resctrl_arch_system_num_rmid_idx();
-	u32 num_closid = resctrl_arch_get_num_closid(r);
 	struct rmid_entry *entry = NULL;
 	int err = 0, i;
 	u32 idx;
 
 	mutex_lock(&rdtgroup_mutex);
-	if (IS_ENABLED(CONFIG_RESCTRL_RMID_DEPENDS_ON_CLOSID)) {
-		u32 *tmp;
-
-		/*
-		 * If the architecture hasn't provided a sanitised value here,
-		 * this may result in larger arrays than necessary. Resctrl will
-		 * use a smaller system wide value based on the resources in
-		 * use.
-		 */
-		tmp = kcalloc(num_closid, sizeof(*tmp), GFP_KERNEL);
-		if (!tmp) {
-			err = -ENOMEM;
-			goto out_unlock;
-		}
-
-		closid_num_dirty_rmid = tmp;
-	}
 
 	rmid_ptrs = kcalloc(idx_limit, sizeof(struct rmid_entry), GFP_KERNEL);
 	if (!rmid_ptrs) {
-		if (IS_ENABLED(CONFIG_RESCTRL_RMID_DEPENDS_ON_CLOSID)) {
-			kfree(closid_num_dirty_rmid);
-			closid_num_dirty_rmid = NULL;
-		}
 		err = -ENOMEM;
 		goto out_unlock;
 	}
@@ -870,11 +879,6 @@ static void dom_data_exit(struct rdt_resource *r)
 	if (!r->mon_capable)
 		goto out_unlock;
 
-	if (IS_ENABLED(CONFIG_RESCTRL_RMID_DEPENDS_ON_CLOSID)) {
-		kfree(closid_num_dirty_rmid);
-		closid_num_dirty_rmid = NULL;
-	}
-
 	kfree(rmid_ptrs);
 	rmid_ptrs = NULL;
 
@@ -938,7 +942,7 @@ bool resctrl_is_mon_event_enabled(enum resctrl_event_id eventid)
 }
 
 /**
  * resctrl_mon_resource_init() - Initialise global monitoring structures.
  *
  * Allocate and initialise global monitor resources that do not belong to a
  * specific domain. i.e. the rmid_ptrs[] used for the limbo and free lists.
@@ -949,7 +953,7 @@ bool resctrl_is_mon_event_enabled(enum resctrl_event_id eventid)
  *
  * Returns 0 for success, or -ENOMEM.
  */
-int resctrl_mon_resource_init(void)
+int resctrl_mon_l3_resource_init(void)
 {
 	struct rdt_resource *r = resctrl_arch_get_resource(RDT_RESOURCE_L3);
 	int ret;
@@ -957,6 +961,10 @@ int resctrl_mon_resource_init(void)
 	if (!r->mon_capable)
 		return 0;
 
+	ret = closid_num_dirty_rmid_init(r);
+	if (ret)
+		return ret;
+
 	ret = dom_data_init(r);
 	if (ret)
 		return ret;
@@ -980,9 +988,10 @@ int resctrl_mon_resource_init(void)
 	return 0;
 }
 
-void resctrl_mon_resource_exit(void)
+void resctrl_mon_l3_resource_exit(void)
 {
 	struct rdt_resource *r = resctrl_arch_get_resource(RDT_RESOURCE_L3);
 
+	closid_num_dirty_rmid_exit();
 	dom_data_exit(r);
 }
@@ -938,7 +942,7 @@ bool resctrl_is_mon_event_enabled(enum resctrl_event_id eventid)
 
 /**
- * resctrl_mon_resource_init() - Initialise global monitoring structures.
+ * resctrl_mon_l3_resource_init() - Initialise global monitoring structures.
  *
  * Allocate and initialise global monitor resources that do not belong to a
-- 
2.50.0


  parent reply	other threads:[~2025-07-11 23:54 UTC|newest]

Thread overview: 61+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2025-07-11 23:53 [PATCH v7 00/31] x86,fs/resctrl telemetry monitoring Tony Luck
2025-07-11 23:53 ` [PATCH v7 01/31] x86,fs/resctrl: Consolidate monitor event descriptions Tony Luck
2025-07-17 17:51   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 02/31] x86,fs/resctrl: Replace architecture event enabled checks Tony Luck
2025-07-11 23:53 ` [PATCH v7 03/31] x86/resctrl: Remove 'rdt_mon_features' global variable Tony Luck
2025-07-11 23:53 ` [PATCH v7 04/31] x86,fs/resctrl: Prepare for more monitor events Tony Luck
2025-07-11 23:53 ` [PATCH v7 05/31] x86,fs/resctrl: Improve domain type checking Tony Luck
2025-07-25 23:17   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 06/31] x86/resctrl: Move L3 initialization into new helper function Tony Luck
2025-07-25 23:21   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 07/31] x86,fs/resctrl: Refactor domain_remove_cpu_mon() ready for new domain types Tony Luck
2025-07-25 23:29   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 08/31] x86/resctrl: Clean up domain_remove_cpu_ctrl() Tony Luck
2025-07-25 23:22   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 09/31] x86,fs/resctrl: Use struct rdt_domain_hdr instead of struct rdt_mon_domain Tony Luck
2025-07-25 23:25   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 10/31] x86,fs/resctrl: Rename struct rdt_mon_domain and rdt_hw_mon_domain Tony Luck
2025-07-25 23:26   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 11/31] x86,fs/resctrl: Rename some L3 specific functions Tony Luck
2025-07-25 23:26   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 12/31] fs/resctrl: Make event details accessible to functions when reading events Tony Luck
2025-07-25 23:27   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 13/31] x86,fs/resctrl: Handle events that can be read from any CPU Tony Luck
2025-07-25 23:32   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 14/31] x86,fs/resctrl: Support binary fixed point event counters Tony Luck
2025-07-25 23:34   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 15/31] x86,fs/resctrl: Add an architectural hook called for each mount Tony Luck
2025-07-25 23:35   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 16/31] x86,fs/resctrl: Add and initialize rdt_resource for package scope core monitor Tony Luck
2025-07-25 23:36   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 17/31] x86/resctrl: Discover hardware telemetry events Tony Luck
2025-07-25 23:39   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 18/31] x86/resctrl: Count valid telemetry aggregators per package Tony Luck
2025-07-25 23:40   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 19/31] x86/resctrl: Complete telemetry event enumeration Tony Luck
2025-07-25 23:41   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 20/31] x86,fs/resctrl: Fill in details of events for guid 0x26696143 and 0x26557651 Tony Luck
2025-07-25 23:43   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 21/31] x86,fs/resctrl: Add architectural event pointer Tony Luck
2025-07-25 23:43   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 22/31] x86/resctrl: Read telemetry events Tony Luck
2025-07-25 23:45   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 23/31] x86/resctrl: Handle domain creation/deletion for RDT_RESOURCE_PERF_PKG Tony Luck
2025-07-25 23:46   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 24/31] x86/resctrl: Add energy/perf choices to rdt boot option Tony Luck
2025-07-25 23:46   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 25/31] x86/resctrl: Handle number of RMIDs supported by telemetry resources Tony Luck
2025-07-25 23:49   ` Reinette Chatre
2025-07-11 23:53 ` Tony Luck [this message]
2025-07-25 23:51   ` [PATCH v7 26/31] fs/resctrl: Fix life-cycle of closid_num_dirty_rmid Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 27/31] x86,fs/resctrl: Move RMID initialization to first mount Tony Luck
2025-07-25 23:53   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 28/31] x86/resctrl: Enable RDT_RESOURCE_PERF_PKG Tony Luck
2025-07-25 23:54   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 29/31] fs/resctrl: Provide interface to create architecture specific debugfs area Tony Luck
2025-07-25 23:55   ` Reinette Chatre
2025-07-11 23:53 ` [PATCH v7 30/31] x86/resctrl: Add debugfs files to show telemetry aggregator status Tony Luck
2025-07-11 23:53 ` [PATCH v7 31/31] x86,fs/resctrl: Update Documentation for package events Tony Luck
2025-07-30 18:42 ` [PATCH v7 00/31] x86,fs/resctrl telemetry monitoring Moger, Babu
2025-07-30 20:27   ` Reinette Chatre
2025-07-30 22:05     ` Moger, Babu

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20250711235341.113933-27-tony.luck@intel.com \
    --to=tony.luck@intel.com \
    --cc=Dave.Martin@arm.com \
    --cc=anil.s.keshavamurthy@intel.com \
    --cc=babu.moger@amd.com \
    --cc=dfustini@baylibre.com \
    --cc=fenghuay@nvidia.com \
    --cc=james.morse@arm.com \
    --cc=linux-kernel@vger.kernel.org \
    --cc=maciej.wieczor-retman@intel.com \
    --cc=patches@lists.linux.dev \
    --cc=peternewman@google.com \
    --cc=reinette.chatre@intel.com \
    --cc=x86@kernel.org \
    --cc=yu.c.chen@intel.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

all inboxes | Powered by JetHome®